﻿-- Author      : Deldinor of Elune
-- Create Date : 2/1/2011 7:31:49 AM

DailyGrind = LibStub("AceAddon-3.0"):NewAddon("DailyGrind", "AceConsole-3.0", "AceEvent-3.0")

local titleColor = "FF70B7FF";
local addonTitle = "|c"..titleColor.."Daily Grind|r";
local blacklistColor = "FF808080";
local blacklistText = "|c"..blacklistColor.."Blacklist|r";
local commandColor = "FFFFC654";
local disabledText = "|cFFFF0000DISABLED|r";
local enabledText = "|cFF00FF00ENABLED|r";
local maxQuests = 25;

function DailyGrind:OnInitialize()   
    self:RegisterChatCommand("dailygrind", "ChatCommand");
    self:RegisterChatCommand("dg", "ChatCommand"); 
    if DailyGrindQuests == nil then
		DailyGrindQuests = { Blacklist = {} };
    end
    if DailyGrindQuests.Blacklist == nil then
		DailyGrindQuests.Blacklist = {};
    end
    if DailyGrindSettings == nil then
		DailyGrindSettings = { Enabled = true; };
    end
end
	
function DailyGrind:OnEnable()
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("QUEST_DETAIL");
    self:RegisterEvent("QUEST_PROGRESS");
    self:RegisterEvent("QUEST_COMPLETE");
end

function DailyGrind:ChatCommand(input)
	local args = { strsplit(" ", input:trim(), 3) };
	local command = args[1];
	
	if command == "import" then
		-- TODO: Implement confirmation.
        for index, value in pairs(KnownDailies) do
			DailyGrindQuests[index] = value;
        end
		DEFAULT_CHAT_FRAME:AddMessage("All known "..addonTitle.." quests imported.");
		
	elseif command == "reset" then
		-- TODO: Implement confirmation.
		DailyGrindQuests = { Blacklist = {} };
		DEFAULT_CHAT_FRAME:AddMessage(addonTitle.." quest history reset.");
		
  	elseif command == "enable" or command == "on" then
		DailyGrindSettings.Enabled = true;
		DEFAULT_CHAT_FRAME:AddMessage(self:GetStatus());
		
	elseif command == "disable" or command == "off" then
		DailyGrindSettings.Enabled = false;
		DEFAULT_CHAT_FRAME:AddMessage(self:GetStatus());
		
	elseif command == "toggle" or command == "tog" then
		DailyGrindSettings.Enabled = not DailyGrindSettings.Enabled;
		DEFAULT_CHAT_FRAME:AddMessage(self:GetStatus());
		
	elseif command == "blacklist" or command == "bl" then
		self:Blacklist(args);
		
	else
		self:ShowHelp();
    end
end

function DailyGrind:GetStatus()
	if DailyGrindSettings.Enabled then
		return addonTitle.." : "..enabledText;
	else
		return addonTitle.." : "..disabledText;
	end
end

function DailyGrind:GetNumPlayerActiveQuests()
	local numEntries, numQuests = GetNumQuestLogEntries();
	return numQuests;
end

function DailyGrind:Blacklist(args)
	if table.getn(args) == 1 then
		DEFAULT_CHAT_FRAME:AddMessage(addonTitle.." : "..blacklistText);
		if self:GetBlacklistCount() > 0 then
			for index, value in pairs(DailyGrindQuests.Blacklist) do
				DEFAULT_CHAT_FRAME:AddMessage("     "..index);
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("     <empty>");
		end
	elseif args[2] ~= nil then
		local action = args[2]:trim();
		local questTitle = "";
		if args[3] ~= nil then
			questTitle = args[3]:trim():gsub("\"", "");			-- get rid of double-quotes (people may assume they're necessary around the quest name)
			questTitle = questTitle:gsub("<", ""):gsub(">", "")	-- and angle brackets, in case people take the documentation too literally...
		end
		if action == "add" then
			self:Blacklist_Add(questTitle);
		elseif action == "remove" or action == "rem" then
			self:Blacklist_Remove(questTitle);
		elseif action == "clear" then
			DailyGrindQuests.Blacklist = {};
			DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." : Cleared");
		end
	else
		self:ShowHelp();
	end
end

-- Encapsulated this because table indexes are case-sensitive.
-- Want to avoid needless user complexity, so removing case-sensitivity.
function DailyGrind:FindBlacklistIndex(questTitle)
	for index, value in pairs(DailyGrindQuests.Blacklist) do
		if index:upper() == questTitle:upper() then
			return index;
		end
	end
	return nil;
end

function DailyGrind:IsBlacklisted(questTitle)
	return self:FindBlacklistIndex(questTitle) ~= nil;
end

-- table.getn() doesn't seem to want to return anything but 0,
-- so just iterating through and counting.
function DailyGrind:GetBlacklistCount()
	local numBlacklists = 0;	
	for index, value in pairs(DailyGrindQuests.Blacklist) do
		numBlacklists = numBlacklists + 1;
	end
	return numBlacklists;
end

function DailyGrind:Blacklist_Add(questTitle)
	if questTitle == "" then
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." needs a quest title to add.");
		return;
	end
	if not self:IsBlacklisted(questTitle) then
		DailyGrindQuests.Blacklist[questTitle] = 1;
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." : Added quest \""..questTitle.."\"");
	else
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." : Quest \""..self:FindBlacklistIndex(questTitle).."\" already exists");
	end
end

function DailyGrind:Blacklist_Remove(questTitle)
	if questTitle == "" then
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." needs a quest title to remove.");
		return;
	end
	if self:IsBlacklisted(questTitle) then
		local index = self:FindBlacklistIndex(questTitle);
		DailyGrindQuests.Blacklist[index] = nil;
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." : Removed quest \""..index.."\"");
	else
		DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." : Quest \""..questTitle.."\" not found");
	end
end

function DailyGrind:GOSSIP_SHOW()
	if not DailyGrindSettings.Enabled then
		return;
	end

	-- Turn in known completed quests.
	local numActiveQuests = GetNumGossipActiveQuests();
	if numActiveQuests > 0 then
		local activeQuests = {GetGossipActiveQuests()};
		for i = 1, numActiveQuests, 1 do
			local titleIdx = (4 * (i - 1)) + 1;
			local isDailyIdx = titleIdx + 3;
			if activeQuests[isDailyIdx] and DailyGrindQuests[activeQuests[titleIdx]] then
				if self:IsBlacklisted(activeQuests[titleIdx]) then
					DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." skipped: "..activeQuests[titleIdx]);
				else
					SelectGossipActiveQuest(i);
				end
			end
		end
	end
	
	-- Accept known available quests.
	local numAvailableQuests = GetNumGossipAvailableQuests();
	if numAvailableQuests > 0 then
		local availableQuests = {GetGossipAvailableQuests()};
		for i = 1, numAvailableQuests, 1 do
			local titleIdx = (5 * (i - 1)) + 1;
			if DailyGrindQuests[availableQuests[titleIdx]] then
				if self:IsBlacklisted(availableQuests[titleIdx]) then
					DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." skipped: "..availableQuests[titleIdx]);
				else
					if self:GetNumPlayerActiveQuests() < maxQuests then
						SelectGossipAvailableQuest(i);
					else
						DEFAULT_CHAT_FRAME:AddMessage(addonTitle.." cannot auto-accept \""..availableQuests[titleIdx].."\": Quest log full.");
					end
				end
			end
		end
	end
	
end

function DailyGrind:QUEST_DETAIL()
	if not DailyGrindSettings.Enabled then
		return;
	end
	local questTitle = GetTitleText();
	if DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		if self:GetNumPlayerActiveQuests() < maxQuests then
			AcceptQuest();
		else
			DEFAULT_CHAT_FRAME:AddMessage(addonTitle.." cannot auto-accept \""..questTitle.."\": Quest log full.");
		end
	end
end

function DailyGrind:QUEST_PROGRESS()
	if not DailyGrindSettings.Enabled then
		return;
	end
	local questTitle = GetTitleText();
	if IsQuestCompletable() and DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		CompleteQuest();
	end
end

function DailyGrind:QUEST_COMPLETE()
	local questTitle = GetTitleText();
	if DailyGrindSettings.Enabled and DailyGrindQuests[questTitle] and not self:IsBlacklisted(questTitle) then
		if GetNumQuestChoices() == 0 then
			GetQuestReward(0)
		end	-- TODO: Implement auto-rewards list
	elseif (QuestIsDaily() or QuestIsWeekly()) then
		DailyGrindQuests[questTitle] = 1;	-- always record, even when disabled
	end
end

function DailyGrind:ShowHelp()
	DEFAULT_CHAT_FRAME:AddMessage("\n==  "..self:GetStatus().."  ==");
	DEFAULT_CHAT_FRAME:AddMessage("All dailies that you complete will be added to a database. Whenever you see that daily again, you will auto-accept/complete it.");
	DEFAULT_CHAT_FRAME:AddMessage("A list of currently-known dailies is included; you may import this list into your database using |c"..commandColor.."/dg import|r.");
	DEFAULT_CHAT_FRAME:AddMessage("Currently, if there is a choice of reward available, you must choose a reward manually. This will be addressed in a future update.");
	DEFAULT_CHAT_FRAME:AddMessage("Usage:  |c"..commandColor.."/dg <enable|on> <disable|off> <tog[gle]> <import> <reset>|r");
	DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." Usage:");
	DEFAULT_CHAT_FRAME:AddMessage(" List: |c"..commandColor.."/dg /bl[acklist]");
	DEFAULT_CHAT_FRAME:AddMessage(" Add: |c"..commandColor.."/dg /bl add <quest title, including punctuation>");
	DEFAULT_CHAT_FRAME:AddMessage(" Remove: |c"..commandColor.."/dg /bl rem[ove] <quest title, including punctuation>");
	DEFAULT_CHAT_FRAME:AddMessage(" Clear: |c"..commandColor.."/dg /bl clear");
	DEFAULT_CHAT_FRAME:AddMessage(blacklistText.." quest titles are NOT case-sensitive.");
end